import pathlib

import pytest

from clinvar_api.msg import (
    Assembly,
    Chromosome,
    ClinicalSignificanceDescription,
    ModeOfInheritance,
)
from clinvar_this import exceptions
from clinvar_this.io.tsv import TsvRecord, read_tsv

DATA_DIR = pathlib.Path(__file__).parent / "data"


def test_read_tsv_path():
    actual = read_tsv(path=DATA_DIR / "example.tsv")
    assert actual == [
        TsvRecord(
            assembly=Assembly.GRCH37,
            chromosome=Chromosome.CHR10,
            pos=115614632,
            ref="A",
            alt="G",
            omim=["OMIM:618278"],
            inheritance=ModeOfInheritance.AUTOSOMAL_RECESSIVE_INHERITANCE,
            clinical_significance_description=ClinicalSignificanceDescription.NOT_PROVIDED,
            local_key="KEY",
            extra_data={"gene": "NHLRC2"},
        )
    ]


def test_read_tsv_file():
    with (DATA_DIR / "example.tsv").open("rt") as inputf:
        actual = read_tsv(file=inputf)
    assert actual == [
        TsvRecord(
            assembly=Assembly.GRCH37,
            chromosome=Chromosome.CHR10,
            pos=115614632,
            ref="A",
            alt="G",
            omim=["OMIM:618278"],
            inheritance=ModeOfInheritance.AUTOSOMAL_RECESSIVE_INHERITANCE,
            clinical_significance_description=ClinicalSignificanceDescription.NOT_PROVIDED,
            local_key="KEY",
            extra_data={"gene": "NHLRC2"},
        )
    ]


def test_read_tsv_path_bad():
    with pytest.raises(exceptions.InvalidFormat):
        read_tsv(path=DATA_DIR / "example.bad.tsv")


def test_read_tsv_error():
    with pytest.raises(TypeError):
        read_tsv()



from unittest.mock import MagicMock, patch

from click.testing import CliRunner

import clinvar_this  # noqa
from clinvar_this import cli, config, exceptions


def test_call_main_help():
    runner = CliRunner()
    result = runner.invoke(cli.cli, ["--help"])
    assert result.exit_code == 0


def test_call_main_verbose():
    runner = CliRunner()
    result = runner.invoke(cli.cli, ["--verbose"])
    assert result.exit_code != 0


def test_call_config():
    runner = CliRunner()
    result = runner.invoke(cli.cli, ["config"])
    assert result.exit_code == 0


def test_call_config_get():
    with patch(
        "clinvar_this.cli.load_config",
        MagicMock(return_value=config.Config(profile="default", auth_token="fake")),
    ):
        runner = CliRunner()
        result = runner.invoke(cli.cli, ["config", "get", "auth_token"])
    assert b"fake" in result.stdout_bytes
    assert result.exit_code == 0


def test_call_config_set_success():
    mock_save_config = MagicMock()
    with patch(
        "clinvar_this.cli.load_config",
        MagicMock(return_value=config.Config(profile="default", auth_token="fake")),
    ), patch("clinvar_this.cli.save_config", mock_save_config):
        runner = CliRunner()
        result = runner.invoke(cli.cli, ["config", "set", "auth_token", "xxx"])
    mock_save_config.assert_called_once_with(
        config.Config(profile="default", auth_token="xxx"), "default"
    )
    assert result.exit_code == 0


def test_call_config_set_success_though_missing_config():
    mock_save_config = MagicMock()
    with patch(
        "clinvar_this.cli.load_config", MagicMock(side_effect=exceptions.ConfigFileMissingException)
    ), patch("clinvar_this.cli.save_config", mock_save_config):
        runner = CliRunner()
        result = runner.invoke(cli.cli, ["config", "set", "auth_token", "xxx"])
    mock_save_config.assert_called_once_with(
        config.Config(profile="default", auth_token="xxx"), "default"
    )
    assert result.exit_code == 0


def test_call_config_set_fail_invalid_name():
    mock_save_config = MagicMock()
    with patch("clinvar_this.cli.save_config", mock_save_config):
        runner = CliRunner()
        result = runner.invoke(cli.cli, ["config", "set", "xxx", "xxx"])
    # assert b"Invalid value" in result.stderr_bytes
    assert result.exit_code != 0



"""Top-level package for ClinVar This!."""

__author__ = """Manuel Holtgrewe"""
__email__ = "manuel.holtgrewe@bih-charite.de"
__version__ = "0.1.0"

from . import _version

__version__ = _version.get_versions()["version"]
